( cd usr/lib64 ; rm -rf libgnutls-extra.so.26 )
( cd usr/lib64 ; ln -sf libgnutls-extra.so.26.11.7 libgnutls-extra.so.26 )
( cd usr/lib64 ; rm -rf libguile-gnutls-v-1.so )
( cd usr/lib64 ; ln -sf libguile-gnutls-v-1.so.0.0.0 libguile-gnutls-v-1.so )
( cd usr/lib64 ; rm -rf libgnutls-extra.so )
( cd usr/lib64 ; ln -sf libgnutls-extra.so.26.11.7 libgnutls-extra.so )
( cd usr/lib64 ; rm -rf libguile-gnutls-v-1.so.0 )
( cd usr/lib64 ; ln -sf libguile-gnutls-v-1.so.0.0.0 libguile-gnutls-v-1.so.0 )
( cd usr/lib64 ; rm -rf libgnutls.so )
( cd usr/lib64 ; ln -sf libgnutls.so.26.11.7 libgnutls.so )
( cd usr/lib64 ; rm -rf libgnutls-openssl.so.26 )
( cd usr/lib64 ; ln -sf libgnutls-openssl.so.26.11.7 libgnutls-openssl.so.26 )
( cd usr/lib64 ; rm -rf libgnutls-openssl.so )
( cd usr/lib64 ; ln -sf libgnutls-openssl.so.26.11.7 libgnutls-openssl.so )
( cd usr/lib64 ; rm -rf libguile-gnutls-extra-v-1.so )
( cd usr/lib64 ; ln -sf libguile-gnutls-extra-v-1.so.0.0.0 libguile-gnutls-extra-v-1.so )
( cd usr/lib64 ; rm -rf libguile-gnutls-extra-v-1.so.0 )
( cd usr/lib64 ; ln -sf libguile-gnutls-extra-v-1.so.0.0.0 libguile-gnutls-extra-v-1.so.0 )
( cd usr/lib64 ; rm -rf libgnutlsxx.so )
( cd usr/lib64 ; ln -sf libgnutlsxx.so.26.11.7 libgnutlsxx.so )
( cd usr/lib64 ; rm -rf libgnutlsxx.so.26 )
( cd usr/lib64 ; ln -sf libgnutlsxx.so.26.11.7 libgnutlsxx.so.26 )
( cd usr/lib64 ; rm -rf libgnutls.so.26 )
( cd usr/lib64 ; ln -sf libgnutls.so.26.11.7 libgnutls.so.26 )
